# Slideshow Shuffle Plugin for xviewer
# Copyright (C) 2008  Johannes Marbach <jm@rapidrabbit.de>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from gi.repository import GObject, Gtk, Xviewer
import random

class SlideshowShufflePlugin(GObject.Object, Xviewer.WindowActivatable):

    # Override XviewerWindowActivatable's window property
    window = GObject.property(type=Xviewer.Window)

    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        random.seed()
        self.slideshow = False
        self.state_handler_id = \
            self.window.connect('window-state-event', self.state_changed_cb, self)

    def do_deactivate(self):
        self.window.disconnect(self.state_handler_id)

    # The callback functions are done statically to avoid causing additional
    # references on the window property causing xviewer to not quit correctly.
    @staticmethod
    def state_changed_cb(window, event, self):
        mode = self.window.get_mode()

        if mode == Xviewer.WindowMode.SLIDESHOW and not self.slideshow:
            # Slideshow starts
            self.slideshow = True

            # Query position of current image
            if not window.get_image():
                pos = 0
            else:
                pos = window.get_store().get_pos_by_image(window.get_image())

            # Generate random map
            uri_list = [row[2].get_uri_for_display() \
                        for row in window.get_store()]
            self.map = {uri_list.pop(pos): 0}
            supply = list(range(1, len(uri_list) + 1))
            for uri in uri_list:
                self.map[uri] = supply.pop(random.randint(0, len(supply) - 1))

            # Put random sort function in place
            self.window.get_store().\
                set_default_sort_func(self.random_sort_function, self)
        elif mode == Xviewer.WindowMode.NORMAL and self.slideshow:
            # Slideshow ends
            self.slideshow = False

            # Put alphabetic sort function in place
            self.window.get_store().\
                set_default_sort_func(self.alphabetic_sort_function)

    @staticmethod
    def random_sort_function(store, iter1, iter2, self):
        pos1 = self.map[store[iter1][2].get_uri_for_display()]
        pos2 = self.map[store[iter2][2].get_uri_for_display()]

        if  pos1 > pos2:
            return 1
        elif pos1 < pos2:
            return -1
        else:
            return 0

    @staticmethod
    def alphabetic_sort_function(store, iter1, iter2, data = None):
        uri1 = store[iter1][2].get_uri_for_display().lower()
        uri2 = store[iter2][2].get_uri_for_display().lower()

        if uri1 > uri2:
            return 1
        elif uri1 < uri2:
            return -1
        else:
            return 0



# -*- coding: utf-8 -*-

# __init__.py -- plugin object
#
# Copyright (C) 2007 - Diego Escalante Urrelo
# Copyright (C) 2006 - Steve Frécinaux
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# Parts from "Interactive Python-GTK Console" (stolen from gedit's python
# console.py which was actually stolen from epiphany's console.py)
#     Copyright (C), 1998 James Henstridge <james@daa.com.au>
#     Copyright (C), 2005 Adam Hooper <adamh@densi.com>
#     Copyrignt (C), 2005 Raphaël Slinckx

from gi.repository import GObject, Gtk, Xviewer, PeasGtk

from .console import PythonConsole
from .config import PythonConsoleConfigWidget

import gettext
_ = gettext.translation('xviewer-plugins', fallback=True).gettext

_UI_STR = """
    <ui>
        <menubar name="MainMenu">
            <menu name="ToolsMenu" action="Tools">
                <separator/>
                <menuitem name="PythonConsole" action="PythonConsole"/>
                <separator/>
            </menu>
        </menubar>
    </ui>
    """


class PythonConsolePlugin(GObject.Object, Xviewer.WindowActivatable, PeasGtk.Configurable):

    # Override XviewerWindowActivatable's window property
    window = GObject.property(type=Xviewer.Window)
    action_group = None

    def __init__(self):
        GObject.Object.__init__(self)
        self.console_window = None
        self.ui_id = 0

    def do_activate(self):
        ui_manager = self.window.get_ui_manager()
        self.action_group = Gtk.ActionGroup(name='PythonConsole')
        self.action_group.add_actions([('PythonConsole', None, \
            _('P_ython Console'), None, None, self.console_cb)], self.window)
        ui_manager.insert_action_group(self.action_group, 0)
        self.ui_id = ui_manager.add_ui_from_string(_UI_STR)

    def do_deactivate(self):
        ui_manager = self.window.get_ui_manager()
        ui_manager.remove_ui(self.ui_id)
        self.ui_id = 0
        ui_manager.remove_action_group(self.action_group)
        self.action_group = None
        ui_manager.ensure_update()
        if self.console_window is not None:
            self.console_window.destroy()

    def console_cb(self, action, window):
        if not self.console_window:
            self.console_window = Gtk.Window()
            console = PythonConsole(namespace = {'__builtins__' : __builtins__,
                                                 'Xviewer' : Xviewer,
                                                 'window' : window})
            console.set_size_request(600, 400)
            console.eval('print("You can access the main window through ' \
                         '\'window\' :\\n%s" % window)', False)
            self.console_window.set_title(_('Python Console'))
            self.console_window.add(console)

            self.console_window.connect('delete-event', self.on_delete_cb)
            self.console_window.show_all()
            self.console_window.set_transient_for(window)
            self.console_window.set_destroy_with_parent(True)
        else:
            self.console_window.show_all()
        self.console_window.grab_focus()

    def on_delete_cb(self, window, event):
        window.destroy()
        self.console_window = None

    def do_create_configure_widget(self):
        config_widget = PythonConsoleConfigWidget(self.plugin_info.get_data_dir())

        return config_widget.configure_widget()
